@@ -8,7 +8,9 @@
 
 OS=`src/helpers/GuessOS`
 case "x$OS" in
-  x*390*) CONFIGPARAM="--with-layout=BinaryDistribution --enable-module=most";;
+  x*OS390*) CONFIGPARAM="--with-layout=BinaryDistribution --enable-module=most";;
+  *cygwin*) CONFIGPARAM="--with-layout=BinaryDistribution --enable-module=most \
+                         --enable-rule=SHARED_CORE --libexecdir=bin";;
       *) CONFIGPARAM="--with-layout=BinaryDistribution --enable-module=most --enable-shared=max";;
 esac
 APDIR=`pwd`
@@ -89,13 +91,12 @@ echo "Creating supplementary files..."
   echo " " && \
   echo "NOTE: Please do not send support-related mails to the address mentioned" && \
   echo "      above or to any member of the Apache Group! Support questions" && \
-  echo "      should be directed to the \"comp.infosystems.www.servers.unix\"" && \
-  echo "      or \"comp.infosystems.www.servers.ms-windows\" newsgroup" && \
-  echo "      (as appropriate for the platform you use), where some of the" && \
-  echo "      Apache team lurk, in the company of many other Apache gurus" && \
-  echo "      who should be able to help." && \
+  echo "      should be directed to the forums mentioned at" && \
+  echo "      http://httpd.apache.org/lists.html#http-users" && \
+  echo "      where some of the Apache team lurk, in the company of many other" && \
+  echo "      Apache gurus who should be able to help." && \
   echo "      If you think you found a bug in Apache or have a suggestion please" && \
-  echo "      visit the bug report page at http://www.apache.org/bug_report.html" && \
+  echo "      visit the bug report page at http://httpd.apache.org/bug_report.html" && \
   echo " " && \
   echo "----------------------------------------------------------------------" && \
   ./bindist/bin/httpd -V && \
@@ -201,7 +202,7 @@ cp README.bindist ../apache_$VER-$OS.README
   echo "lmkdir \$SR/proxy 750" && \
   echo "lmkdir \$SR/logs 750" && \
   echo "lcopy bindist/man \$SR/man 755 644" && \
-  echo "lcopy bindist/libexec \$SR/libexec 750 644" && \
+  echo "lcopy bindist/libexec \$SR/libexec 750 750" && \
   echo "lcopy bindist/include \$SR/include 755 644" && \
   echo "lcopy bindist/icons \$SR/icons 755 644" && \
   echo "lcopy bindist/cgi-bin \$SR/cgi-bin 750 750" && \
@@ -272,7 +273,7 @@ else
     if [ "x$TAR" != "x" ]
     then
       case "x$OS" in
-        x*390*) $TAR -cfU ../apache_$VER-$OS.tar -C .. apache_$VER;;
+        x*OS390*) $TAR -cfU ../apache_$VER-$OS.tar -C .. apache_$VER;;
 	    *) (cd .. && $TAR -cf apache_$VER-$OS.tar apache_$VER);;
       esac
       if [ "x$GZIP" != "x" ]
