@@ -2,8 +2,7 @@
 ##
 ##  fmn.sh -- find a modules (structure) name
 ##
-##  Extracted from the Configure script by
-##  Ralf S. Engelschall <rse@apache.org> for use with 
+##  Extracted from the Configure script for use with
 ##  Apache's Autoconf-style Interface (APACI).
 ##
 #
@@ -15,7 +14,7 @@
 modfile=$1
 
 #   the part from the Configure script
-tmpfile=/tmp/fmn.tmp.$$
+tmpfile=${TMPDIR-/tmp}/fmn.tmp.$$
 rm -f $tmpfile
 modname=''
 ext=`echo $modfile | sed 's/^.*\.//'`
