@@ -1,7 +1,7 @@
 /* ====================================================================
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 2000-2002 The Apache Software Foundation.  All rights
+ * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -284,15 +284,19 @@ typedef int rlim_t;
 #define HAVE_FCNTL_SERIALIZED_ACCEPT
 #define HAVE_SYSVSEM_SERIALIZED_ACCEPT
 #define NEED_UNION_SEMUN
-#if !defined(USE_PTHREAD_SERIALIZED_ACCEPT)
+#if AIX >= 430
+#define HAVE_PTHREAD_SERIALIZED_ACCEPT
+#endif
 #define USE_FCNTL_SERIALIZED_ACCEPT
+#if AIX >= 432
+#define SINGLE_LISTEN_UNSERIALIZED_ACCEPT 
 #endif
 #ifdef USEBCOPY
 #define memmove(a,b,c) bcopy(b,a,c)
 #endif
-#if AIX >= 51
+#if AIX >= 510
 #define NET_SIZE_T socklen_t
-#elif AIX >= 42
+#elif AIX >= 420
 #define NET_SIZE_T size_t
 #endif
 
@@ -962,10 +966,11 @@ typedef int rlim_t;
 #define NO_SETSID
 #define NO_SLACK
 #define NO_TIMES
+#ifndef TPF_HAVE_SIGACTION
 #define NO_USE_SIGACTION
+#endif
 #define USE_LONGJMP
 #define USE_SHMGET_SCOREBOARD
-/*#define USE_TPF_SCOREBOARD*/
 #define USE_TPF_ACCEPT
 #define HAVE_TPF_CORE_SERIALIZED_ACCEPT
 #define USE_TPF_SELECT
@@ -990,6 +995,7 @@ typedef int rlim_t;
 #define USE_MMAP_FILES
 #define NEED_UNION_SEMUN
 #define HAVE_SYSVSEM_SERIALIZED_ACCEPT
+#define HAVE_FCNTL_SERIALIZED_ACCEPT
 #define _POSIX_SOURCE
 #include <signal.h>
 #ifdef SIGDUMP  /* SIGDUMP is not defined by OS/390 v1r2 */
@@ -1024,7 +1030,7 @@ typedef int rlim_t;
 #define HAVE_FCNTL_SERIALIZED_ACCEPT
 #define HAVE_PTHREAD_SERIALIZED_ACCEPT
 #define SINGLE_LISTEN_UNSERIALIZED_ACCEPT
-#if !defined(USE_FNCTL_SERIALIZED_ACCEPT)
+#if !defined(USE_FCNTL_SERIALIZED_ACCEPT)
 #define USE_PTHREAD_SERIALIZED_ACCEPT
 #endif
 
