@@ -1,58 +1,59 @@
 /* ====================================================================
- * Copyright (c) 1995-1999 The Apache Group.  All rights reserved.
+ * The Apache Software License, Version 1.1
+ *
+ * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
+ * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
+ *    notice, this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in
  *    the documentation and/or other materials provided with the
  *    distribution.
  *
- * 3. All advertising materials mentioning features or use of this
- *    software must display the following acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
+ * 3. The end-user documentation included with the redistribution,
+ *    if any, must include the following acknowledgment:
+ *       "This product includes software developed by the
+ *        Apache Software Foundation (http://www.apache.org/)."
+ *    Alternately, this acknowledgment may appear in the software itself,
+ *    if and wherever such third-party acknowledgments normally appear.
  *
- * 4. The names "Apache Server" and "Apache Group" must not be used to
- *    endorse or promote products derived from this software without
- *    prior written permission. For written permission, please contact
- *    apache@apache.org.
+ * 4. The names "Apache" and "Apache Software Foundation" must
+ *    not be used to endorse or promote products derived from this
+ *    software without prior written permission. For written
+ *    permission, please contact apache@apache.org.
  *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
+ * 5. Products derived from this software may not be called "Apache",
+ *    nor may "Apache" appear in their name, without prior written
+ *    permission of the Apache Software Foundation.
  *
- * 6. Redistributions of any form whatsoever must retain the following
- *    acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
- *
- * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
- * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
  * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
- * OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
+ * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
+ * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  * ====================================================================
  *
  * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Group and was originally based
- * on public domain software written at the National Center for
- * Supercomputing Applications, University of Illinois, Urbana-Champaign.
- * For more information on the Apache Group and the Apache HTTP server
- * project, please see <http://www.apache.org/>.
+ * individuals on behalf of the Apache Software Foundation.  For more
+ * information on the Apache Software Foundation, please see
+ * <http://www.apache.org/>.
  *
+ * Portions of this software are based upon public domain software
+ * originally written at the National Center for Supercomputing Applications,
+ * University of Illinois, Urbana-Champaign.
  */
 
 /*
@@ -75,9 +76,9 @@
  */
 #define ap_isascii(c) ((OS_ASC(c) & 0x80) == 0)
 
-typedef struct handlers_info {
+typedef struct attrib_info {
     char *name;
-} handlers_info;
+} attrib_info;
 
 typedef struct {
     table *forced_types;        /* Additional AddTyped stuff */
@@ -85,7 +86,9 @@ typedef struct {
     table *charset_types;	/* Added with AddCharset... */
     table *language_types;      /* Added with AddLanguage... */
     table *handlers;            /* Added with AddHandler...  */
-    array_header *handlers_remove;     /* List of handlers to remove */
+    array_header *handlers_remove; /* List of handlers to remove */
+    array_header *types_remove;	/* List of MIME types to remove */
+    array_header *encodings_remove; /* List of encodings to remove */
 
     char *type;                 /* Type forced with ForceType  */
     char *handler;              /* Handler forced with SetHandler */
@@ -122,7 +125,9 @@ static void *create_mime_dir_config(pool *p, char *dummy)
     new->charset_types = ap_make_table(p, 4);
     new->language_types = ap_make_table(p, 4);
     new->handlers = ap_make_table(p, 4);
-    new->handlers_remove = ap_make_array(p, 4, sizeof(handlers_info));
+    new->handlers_remove = ap_make_array(p, 4, sizeof(attrib_info));
+    new->types_remove = ap_make_array(p, 4, sizeof(attrib_info));
+    new->encodings_remove = ap_make_array(p, 4, sizeof(attrib_info));
 
     new->type = NULL;
     new->handler = NULL;
@@ -138,23 +143,31 @@ static void *merge_mime_dir_configs(pool *p, void *basev, void *addv)
     mime_dir_config *new =
         (mime_dir_config *) ap_palloc(p, sizeof(mime_dir_config));
     int i;
-    handlers_info *hand;
-
-    hand = (handlers_info *) add->handlers_remove->elts;
-    for (i = 0; i < add->handlers_remove->nelts; i++) {
-        ap_table_unset(base->handlers, hand[i].name);
-    }
+    attrib_info *suffix;
 
     new->forced_types = ap_overlay_tables(p, add->forced_types,
-					 base->forced_types);
+					  base->forced_types);
     new->encoding_types = ap_overlay_tables(p, add->encoding_types,
-                                         base->encoding_types);
+					    base->encoding_types);
     new->charset_types = ap_overlay_tables(p, add->charset_types,
 					   base->charset_types);
     new->language_types = ap_overlay_tables(p, add->language_types,
-                                         base->language_types);
+					    base->language_types);
     new->handlers = ap_overlay_tables(p, add->handlers,
-                                   base->handlers);
+				      base->handlers);
+
+    suffix = (attrib_info *) add->handlers_remove->elts;
+    for (i = 0; i < add->handlers_remove->nelts; i++) {
+        ap_table_unset(new->handlers, suffix[i].name);
+    }
+    suffix = (attrib_info *) add->types_remove->elts;
+    for (i = 0; i < add->types_remove->nelts; i++) {
+        ap_table_unset(new->forced_types, suffix[i].name);
+    }
+    suffix = (attrib_info *) add->encodings_remove->elts;
+    for (i = 0; i < add->encodings_remove->nelts; i++) {
+        ap_table_unset(new->encoding_types, suffix[i].name);
+    }
 
     new->type = add->type ? add->type : base->type;
     new->handler = add->handler ? add->handler : base->handler;
@@ -225,13 +238,47 @@ static const char *add_handler(cmd_parms *cmd, mime_dir_config *m, char *hdlr,
 static const char *remove_handler(cmd_parms *cmd, void *m, char *ext)
 {
     mime_dir_config *mcfg = (mime_dir_config *) m;
-    handlers_info *hand;
+    attrib_info *suffix;
 
     if (*ext == '.') {
         ++ext;
     }
-    hand = (handlers_info *) ap_push_array(mcfg->handlers_remove);
-    hand->name = ap_pstrdup(cmd->pool, ext);
+    suffix = (attrib_info *) ap_push_array(mcfg->handlers_remove);
+    suffix->name = ap_pstrdup(cmd->pool, ext);
+    return NULL;
+}
+
+/*
+ * Just like the previous function, except that it records encoding
+ * associations to be undone.
+ */
+static const char *remove_encoding(cmd_parms *cmd, void *m, char *ext)
+{
+    mime_dir_config *mcfg = (mime_dir_config *) m;
+    attrib_info *suffix;
+
+    if (*ext == '.') {
+        ++ext;
+    }
+    suffix = (attrib_info *) ap_push_array(mcfg->encodings_remove);
+    suffix->name = ap_pstrdup(cmd->pool, ext);
+    return NULL;
+}
+
+/*
+ * Similar to the previous functions, except that it deals with filename
+ * suffix/MIME-type associations.
+ */
+static const char *remove_type(cmd_parms *cmd, void *m, char *ext)
+{
+    mime_dir_config *mcfg = (mime_dir_config *) m;
+    attrib_info *suffix;
+
+    if (*ext == '.') {
+        ++ext;
+    }
+    suffix = (attrib_info *) ap_push_array(mcfg->types_remove);
+    suffix->name = ap_pstrdup(cmd->pool, ext);
     return NULL;
 }
 
@@ -262,6 +309,10 @@ static const command_rec mime_cmds[] =
      "a media type"},
     {"RemoveHandler", remove_handler, NULL, OR_FILEINFO, ITERATE,
      "one or more file extensions"},
+    {"RemoveEncoding", remove_encoding, NULL, OR_FILEINFO, ITERATE,
+     "one or more file extensions"},
+    {"RemoveType", remove_type, NULL, OR_FILEINFO, ITERATE,
+     "one or more file extensions"},
     {"SetHandler", ap_set_string_slot_lower, 
      (void *)XtOffsetOf(mime_dir_config, handler), OR_FILEINFO, TAKE1, 
      "a handler name"},
@@ -341,7 +392,7 @@ static char *zap_sp(char *s)
     return (s);
 }
 
-static int is_token(char c)
+static int is_token(int c)
 {
     int res;
 
@@ -350,7 +401,7 @@ static int is_token(char c)
     return res;
 }
 
-static int is_qtext(char c)
+static int is_qtext(int c)
 {
     int res;
 
@@ -564,27 +615,38 @@ static content_type *analyze_ct(pool *p, char *s)
 
 static int find_ct(request_rec *r)
 {
-    const char *fn = strrchr(r->filename, '/');
-    mime_dir_config *conf =
-    (mime_dir_config *) ap_get_module_config(r->per_dir_config, &mime_module);
+    mime_dir_config *conf;
+    array_header *exception_list;
+    const char *fn;
     char *ext;
-    const char *orighandler = r->handler;
     const char *type;
     const char *charset = NULL;
+    int found_metadata = 0;
 
     if (S_ISDIR(r->finfo.st_mode)) {
         r->content_type = DIR_MAGIC_TYPE;
         return OK;
     }
 
-    /* TM -- FIXME
-     * if r->filename does not contain a '/', the following passes a null
-     * pointer to getword, causing a SEGV ..
-     */
+    conf = (mime_dir_config *) ap_get_module_config(r->per_dir_config, 
+                                                    &mime_module);
 
-    if (fn == NULL) {
+    exception_list = ap_make_array(r->pool, 2, sizeof(char *));
+
+    /* Always drop the leading element */
+    fn = strrchr(r->filename, '/');
+    if (fn == NULL)
 	fn = r->filename;
-    }
+    else
+        ++fn;
+
+    /* The exception list keeps track of those filename components that
+     * are not associated with extensions indicating metadata.
+     * The base name is always the first exception (i.e., "txt.html" has
+     * a basename of "txt" even though it might look like an extension).
+     */
+    ext = ap_getword(r->pool, &fn, '.');
+    *((const char **) ap_push_array(exception_list)) = ext;
 
     /* Parse filename extensions, which can be in any order */
     while ((ext = ap_getword(r->pool, &fn, '.')) && *ext) {
@@ -632,19 +694,19 @@ static int find_ct(request_rec *r)
             found = 1;
         }
 
-        /* This is to deal with cases such as foo.gif.bak, which we want
-         * to not have a type. So if we find an unknown extension, we
-         * zap the type/language/encoding and reset the handler
-         */
+        if (found)
+            found_metadata = 1;
+        else
+            *((const char **) ap_push_array(exception_list)) = ext;
+    }
 
-        if (!found) {
-            r->content_type = NULL;
-            r->content_language = NULL;
-            r->content_languages = NULL;
-            r->content_encoding = NULL;
-            r->handler = orighandler;
-	    charset = NULL;
-	}
+    /* Need to see a notes entry on r for unrecognized elements.
+     * Somebody better claim them! If we did absolutly nothing,
+     * skip the notes to alert mod_negotiation we are clueless.
+     */
+    if (found_metadata) {
+        ap_table_setn(r->notes, "ap-mime-exceptions-list",
+                      (void *) exception_list);
     }
 
     if (r->content_type) {
