@@ -1,7 +1,7 @@
 /* ====================================================================
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 2000-2002 The Apache Software Foundation.  All rights
+ * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -91,6 +91,8 @@
  *
  * Special values for 'name' are:
  *
+ *   server_addr       IP address of interface on which request arrived
+ *                     (analogous to SERVER_ADDR set in ap_add_common_vars())
  *   remote_host        Remote host name (if available)
  *   remote_addr        Remote IP address
  *   remote_user        Remote authenticated user (if any)
@@ -127,7 +129,8 @@ enum special {
     SPECIAL_REMOTE_USER,
     SPECIAL_REQUEST_URI,
     SPECIAL_REQUEST_METHOD,
-    SPECIAL_REQUEST_PROTOCOL
+    SPECIAL_REQUEST_PROTOCOL,
+    SPECIAL_SERVER_ADDR
 };
 typedef struct {
     char *name;                 /* header name */
@@ -279,6 +282,9 @@ static const char *add_setenvif_core(cmd_parms *cmd, void *mconfig,
 	else if (!strcasecmp(fname, "request_protocol")) {
 	    new->special_type = SPECIAL_REQUEST_PROTOCOL;
 	}
+        else if (!strcasecmp(fname, "server_addr")) {
+            new->special_type = SPECIAL_SERVER_ADDR;
+        }
 	else {
 	    new->special_type = SPECIAL_NOT;
 	}
@@ -396,7 +402,10 @@ static int match_headers(request_rec *r)
 	    switch (b->special_type) {
 	    case SPECIAL_REMOTE_ADDR:
 		val = r->connection->remote_ip;
-		break;
+                break;
+            case SPECIAL_SERVER_ADDR:
+                val = r->connection->local_ip;
+                break;
 	    case SPECIAL_REMOTE_HOST:
 		val =  ap_get_remote_host(r->connection, r->per_dir_config,
 					  REMOTE_NAME);
