@@ -1,5 +1,8 @@
 /* ====================================================================
- * Copyright (c) 1995-1999 The Apache Group.  All rights reserved.
+ * The Apache Software License, Version 1.1
+ *
+ * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
+ * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -13,46 +16,44 @@
  *    the documentation and/or other materials provided with the
  *    distribution.
  *
- * 3. All advertising materials mentioning features or use of this
- *    software must display the following acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
+ * 3. The end-user documentation included with the redistribution,
+ *    if any, must include the following acknowledgment:
+ *       "This product includes software developed by the
+ *        Apache Software Foundation (http://www.apache.org/)."
+ *    Alternately, this acknowledgment may appear in the software itself,
+ *    if and wherever such third-party acknowledgments normally appear.
  *
- * 4. The names "Apache Server" and "Apache Group" must not be used to
- *    endorse or promote products derived from this software without
- *    prior written permission. For written permission, please contact
- *    apache@apache.org.
+ * 4. The names "Apache" and "Apache Software Foundation" must
+ *    not be used to endorse or promote products derived from this
+ *    software without prior written permission. For written
+ *    permission, please contact apache@apache.org.
  *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
+ * 5. Products derived from this software may not be called "Apache",
+ *    nor may "Apache" appear in their name, without prior written
+ *    permission of the Apache Software Foundation.
  *
- * 6. Redistributions of any form whatsoever must retain the following
- *    acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
- *
- * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
- * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
  * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
- * OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
+ * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
+ * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  * ====================================================================
  *
  * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Group and was originally based
- * on public domain software written at the National Center for
- * Supercomputing Applications, University of Illinois, Urbana-Champaign.
- * For more information on the Apache Group and the Apache HTTP server
- * project, please see <http://www.apache.org/>.
+ * individuals on behalf of the Apache Software Foundation.  For more
+ * information on the Apache Software Foundation, please see
+ * <http://www.apache.org/>.
  *
+ * Portions of this software are based upon public domain software
+ * originally written at the National Center for Supercomputing Applications,
+ * University of Illinois, Urbana-Champaign.
  */
 
 /*
@@ -67,14 +68,13 @@
 #include "http_log.h"
 #include "multithread.h"
 
-#ifdef MULTITHREAD
-#error sorry this module does not support multithreaded servers yet
-#endif
-
 typedef struct {
     unsigned int stamp;
     unsigned int in_addr;
     unsigned int pid;
+#ifdef MULTITHREAD
+    unsigned int tid;
+#endif
     unsigned short counter;
 } unique_id_rec;
 
@@ -91,8 +91,7 @@ typedef struct {
  *
  * We also further assume that pids fit in 32-bits.  If something uses more
  * than 32-bits, the fix is trivial, but it requires the unrolled uuencoding
- * loop to be extended.  * A similar fix is needed to support multithreaded
- * servers, using a pid/tid combo.
+ * loop to be extended.
  *
  * Together, the in_addr and pid are assumed to absolutely uniquely identify
  * this one child from all other currently running children on all servers
@@ -129,26 +128,87 @@ typedef struct {
 /*
  * Sun Jun  7 05:43:49 CEST 1998 -- Alvaro
  * More comments:
- * 1) The UUencoding prodecure is now done in a general way, avoiding the problems
- * with sizes and paddings that can arise depending on the architecture. Now the
- * offsets and sizes of the elements of the unique_id_rec structure are calculated
- * in unique_id_global_init; and then used to duplicate the structure without the
- * paddings that might exist. The multithreaded server fix should be now very easy:
- * just add a new "tid" field to the unique_id_rec structure, and increase by one
- * UNIQUE_ID_REC_MAX.
- * 2) unique_id_rec.stamp has been changed from "time_t" to "unsigned int", because
- * its size is 64bits on some platforms (linux/alpha), and this caused problems with
- * htonl/ntohl. Well, this shouldn't be a problem till year 2106.
+ * 1) The UUencoding prodecure is now done in a general way, avoiding
+ * the problems with sizes and paddings that can arise depending on
+ * the architecture. Now the offsets and sizes of the elements of the
+ * unique_id_rec structure are calculated in unique_id_global_init;
+ * and then used to duplicate the structure without the paddings that
+ * might exist. The multithreaded server fix should be now very easy:
+ * just add a new "tid" field to the unique_id_rec structure, and
+ * increase by one UNIQUE_ID_REC_MAX.
+ * 2) unique_id_rec.stamp has been changed from "time_t" to
+ * "unsigned int", because its size is 64bits on some platforms
+ * (linux/alpha), and this caused problems with htonl/ntohl. Well,
+ * this shouldn't be a problem till year 2106.
  */
 
 static unsigned global_in_addr;
 
-static APACHE_TLS unique_id_rec cur_unique_id;
+#ifdef WIN32
+
+static DWORD tls_index;
+
+BOOL WINAPI DllMain (HINSTANCE dllhandle, DWORD reason, LPVOID reserved)
+{
+    LPVOID memptr;
+
+    switch (reason) {
+    case DLL_PROCESS_ATTACH:
+	tls_index = TlsAlloc();
+    case DLL_THREAD_ATTACH: /* intentional no break */
+	TlsSetValue(tls_index, calloc(sizeof(unique_id_rec), 1));
+	break;
+    case DLL_THREAD_DETACH:
+	memptr = TlsGetValue(tls_index);
+	if (memptr) {
+	    free (memptr);
+	    TlsSetValue (tls_index, 0);
+	}
+	break;
+    }
+
+    return TRUE;
+}
+
+static unique_id_rec* get_cur_unique_id(int parent)
+{
+    /* Apache initializes the child process, not the individual child threads.
+     * Copy the original parent record if this->pid is not yet initialized.
+     */
+    static unique_id_rec *parent_id;
+    unique_id_rec *cur_unique_id = (unique_id_rec *) TlsGetValue(tls_index);
+
+    if (parent) {
+        parent_id = cur_unique_id;
+    }
+    else if (!cur_unique_id->pid) {
+        memcpy(cur_unique_id, parent_id, sizeof(*parent_id));
+    }
+    return cur_unique_id;
+}
+
+#else /* !WIN32 */
+
+/* Even when not MULTITHREAD, this will return a single structure, since
+ * APACHE_TLS should be defined as empty on single-threaded platforms.
+ */
+static unique_id_rec* get_cur_unique_id(int parent)
+{
+    static APACHE_TLS unique_id_rec spcid;
+    return &spcid;
+}
+
+#endif /* !WIN32 */
+
 
 /*
  * Number of elements in the structure unique_id_rec.
  */
+#ifdef MULTITHREAD
+#define UNIQUE_ID_REC_MAX 5
+#else
 #define UNIQUE_ID_REC_MAX 4
+#endif
 
 static unsigned short unique_id_rec_offset[UNIQUE_ID_REC_MAX],
                       unique_id_rec_size[UNIQUE_ID_REC_MAX],
@@ -165,20 +225,31 @@ static void unique_id_global_init(server_rec *s, pool *p)
 #ifndef NO_GETTIMEOFDAY
     struct timeval tv;
 #endif
+    unique_id_rec *cur_unique_id = get_cur_unique_id(1);
 
     /*
      * Calculate the sizes and offsets in cur_unique_id.
      */
     unique_id_rec_offset[0] = XtOffsetOf(unique_id_rec, stamp);
-    unique_id_rec_size[0] = sizeof(cur_unique_id.stamp);
+    unique_id_rec_size[0] = sizeof(cur_unique_id->stamp);
     unique_id_rec_offset[1] = XtOffsetOf(unique_id_rec, in_addr);
-    unique_id_rec_size[1] = sizeof(cur_unique_id.in_addr);
+    unique_id_rec_size[1] = sizeof(cur_unique_id->in_addr);
     unique_id_rec_offset[2] = XtOffsetOf(unique_id_rec, pid);
-    unique_id_rec_size[2] = sizeof(cur_unique_id.pid);
+    unique_id_rec_size[2] = sizeof(cur_unique_id->pid);
+#ifdef MULTITHREAD
+    unique_id_rec_offset[3] = XtOffsetOf(unique_id_rec, tid);
+    unique_id_rec_size[3] = sizeof(cur_unique_id->tid);
+    unique_id_rec_offset[4] = XtOffsetOf(unique_id_rec, counter);
+    unique_id_rec_size[4] = sizeof(cur_unique_id->counter);
+    unique_id_rec_total_size = unique_id_rec_size[0] + unique_id_rec_size[1]
+                             + unique_id_rec_size[2] + unique_id_rec_size[3]
+                             + unique_id_rec_size[4];
+#else
     unique_id_rec_offset[3] = XtOffsetOf(unique_id_rec, counter);
-    unique_id_rec_size[3] = sizeof(cur_unique_id.counter);
-    unique_id_rec_total_size = unique_id_rec_size[0] + unique_id_rec_size[1] +
-                               unique_id_rec_size[2] + unique_id_rec_size[3];
+    unique_id_rec_size[3] = sizeof(cur_unique_id->counter);
+    unique_id_rec_total_size = unique_id_rec_size[0] + unique_id_rec_size[1]
+                             + unique_id_rec_size[2] + unique_id_rec_size[3];
+#endif
 
     /*
      * Calculate the size of the structure when encoded.
@@ -192,22 +263,23 @@ static void unique_id_global_init(server_rec *s, pool *p)
      */
     if (gethostname(str, sizeof(str) - 1) != 0) {
         ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ALERT, s,
-          "gethostname: mod_unique_id requires the hostname of the server");
+		     "gethostname: mod_unique_id requires the "
+		     "hostname of the server");
         exit(1);
     }
     str[sizeof(str) - 1] = '\0';
 
     if ((hent = gethostbyname(str)) == NULL) {
         ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ALERT, s,
-                    "mod_unique_id: unable to gethostbyname(\"%s\")", str);
+		     "mod_unique_id: unable to gethostbyname(\"%s\")", str);
         exit(1);
     }
 
     global_in_addr = ((struct in_addr *) hent->h_addr_list[0])->s_addr;
 
     ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_INFO, s,
-                "mod_unique_id: using ip addr %s",
-                inet_ntoa(*(struct in_addr *) hent->h_addr_list[0]));
+		 "mod_unique_id: using ip addr %s",
+		 inet_ntoa(*(struct in_addr *) hent->h_addr_list[0]));
 
     /*
      * If the server is pummelled with restart requests we could possibly end
@@ -242,18 +314,16 @@ static void unique_id_child_init(server_rec *s, pool *p)
 #ifndef NO_GETTIMEOFDAY
     struct timeval tv;
 #endif
+    unique_id_rec *cur_unique_id = get_cur_unique_id(1);
 
     /*
      * Note that we use the pid because it's possible that on the same
      * physical machine there are multiple servers (i.e. using Listen). But
      * it's guaranteed that none of them will share the same pids between
      * children.
-     * 
-     * XXX: for multithread this needs to use a pid/tid combo and probably
-     * needs to be expanded to 32 bits
      */
     pid = getpid();
-    cur_unique_id.pid = pid;
+    cur_unique_id->pid = pid;
 
     /*
      * Test our assumption that the pid is 32-bits.  It's possible that
@@ -261,12 +331,12 @@ static void unique_id_child_init(server_rec *s, pool *p)
      * of them.  It would have been really nice to test this during
      * global_init ... but oh well.
      */
-    if (cur_unique_id.pid != pid) {
+    if ((pid_t)cur_unique_id->pid != pid) {
         ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_CRIT, s,
-                    "oh no! pids are greater than 32-bits!  I'm broken!");
+		     "oh no! pids are greater than 32-bits!  I'm broken!");
     }
 
-    cur_unique_id.in_addr = global_in_addr;
+    cur_unique_id->in_addr = global_in_addr;
 
     /*
      * If we use 0 as the initial counter we have a little less protection
@@ -275,16 +345,16 @@ static void unique_id_child_init(server_rec *s, pool *p)
      */
 #ifndef NO_GETTIMEOFDAY
     if (gettimeofday(&tv, NULL) == -1) {
-        cur_unique_id.counter = 0;
+        cur_unique_id->counter = 0;
     }
     else {
 	/* Some systems have very low variance on the low end of their
 	 * system counter, defend against that.
 	 */
-        cur_unique_id.counter = tv.tv_usec / 10;
+        cur_unique_id->counter = tv.tv_usec / 10;
     }
 #else
-    cur_unique_id.counter = 0;
+    cur_unique_id->counter = 0;
 #endif
 
     /*
@@ -292,8 +362,8 @@ static void unique_id_child_init(server_rec *s, pool *p)
      * identifiers are comparable between machines of different byte
      * orderings.  Note in_addr is already in network order.
      */
-    cur_unique_id.pid = htonl(cur_unique_id.pid);
-    cur_unique_id.counter = htons(cur_unique_id.counter);
+    cur_unique_id->pid = htonl(cur_unique_id->pid);
+    cur_unique_id->counter = htons(cur_unique_id->counter);
 }
 
 /* NOTE: This is *NOT* the same encoding used by base64encode ... the last two
@@ -324,30 +394,45 @@ static int gen_unique_id(request_rec *r)
     unsigned short counter;
     const char *e;
     int i,j,k;
+    unique_id_rec *cur_unique_id = get_cur_unique_id(0);
 
     /* copy the unique_id if this is an internal redirect (we're never
      * actually called for sub requests, so we don't need to test for
      * them) */
-    if (r->prev && (e = ap_table_get(r->subprocess_env, "REDIRECT_UNIQUE_ID"))) {
+    if (r->prev
+	&& (e = ap_table_get(r->subprocess_env, "REDIRECT_UNIQUE_ID"))) {
 	ap_table_setn(r->subprocess_env, "UNIQUE_ID", e);
 	return DECLINED;
     }
 
-    cur_unique_id.stamp = htonl((unsigned int)r->request_time);
+    cur_unique_id->stamp = htonl((unsigned int)r->request_time);
+
+#ifdef MULTITHREAD
+    /*
+     * Note that we use the pid because it's possible that on the same
+     * physical machine there are multiple servers (i.e. using Listen). But
+     * it's guaranteed that none of them will share the same pid+tids between
+     * children.
+     */
+    cur_unique_id->tid = gettid();
+    cur_unique_id->tid = htonl(cur_unique_id->tid);
+#endif
 
     /* we'll use a temporal buffer to avoid uuencoding the possible internal
-     * paddings of the original structure */
+     * paddings of the original structure
+     */
     x = (unsigned char *) &paddedbuf;
-    y = (unsigned char *) &cur_unique_id;
+    y = (unsigned char *) cur_unique_id;
     k = 0;
     for (i = 0; i < UNIQUE_ID_REC_MAX; i++) {
-        y = ((unsigned char *) &cur_unique_id) + unique_id_rec_offset[i];
+        y = ((unsigned char *) cur_unique_id) + unique_id_rec_offset[i];
         for (j = 0; j < unique_id_rec_size[i]; j++, k++) {
             x[k] = y[j];
         }
     }
     /*
-     * We reset two more bytes just in case padding is needed for the uuencoding.
+     * We reset two more bytes just in case padding is needed for
+     * the uuencoding.
      */
     x[k++] = '\0';
     x[k++] = '\0';
@@ -359,9 +444,13 @@ static int gen_unique_id(request_rec *r)
         y = x + i;
         str[k++] = uuencoder[y[0] >> 2];
         str[k++] = uuencoder[((y[0] & 0x03) << 4) | ((y[1] & 0xf0) >> 4)];
-        if (k == unique_id_rec_size_uu) break;
+        if (k == unique_id_rec_size_uu) {
+	    break;
+	}
         str[k++] = uuencoder[((y[1] & 0x0f) << 2) | ((y[2] & 0xc0) >> 6)];
-        if (k == unique_id_rec_size_uu) break;
+        if (k == unique_id_rec_size_uu) {
+	    break;
+	}
         str[k++] = uuencoder[y[2] & 0x3f];
     }
     str[k++] = '\0';
@@ -370,13 +459,12 @@ static int gen_unique_id(request_rec *r)
     ap_table_setn(r->subprocess_env, "UNIQUE_ID", str);
 
     /* and increment the identifier for the next call */
-    counter = ntohs(cur_unique_id.counter) + 1;
-    cur_unique_id.counter = htons(counter);
+    counter = ntohs(cur_unique_id->counter) + 1;
+    cur_unique_id->counter = htons(counter);
 
     return DECLINED;
 }
 
-
 module MODULE_VAR_EXPORT unique_id_module = {
     STANDARD_MODULE_STUFF,
     unique_id_global_init,      /* initializer */
